#pragma once
#if !defined(ALBASEUSCONV_ICONV_HPP_)
#define ALBASEUSCONV_ICONV_HPP_

/**@file
   UNICODE 文字列型の定義、及び、
   基本的なコード変換クラス（CBaseUSConv）の宣言及び実装。

   @note ICONV バージョン
**/

#include <cstring>
#include <string>
#include <climits>
#include <iconv.h>

//#define CONST_CAST(type)	const_cast<type>(var)
#define CONST_CAST(type,var)	var


namespace al {

typedef unsigned char	uint8_type;

#if SHRT_MAX == 32767
typedef unsigned short	uint16_type;
#endif

#if LONG_MAX == 2147483647L
typedef unsigned long	uint32_type;
#elif INT_MAX == 2147483647
typedef unsigned int	uint32_type;
#endif

typedef std::basic_string<uint8_type>		u8_string;
typedef std::basic_string<uint16_type>		u16_string;
typedef std::basic_string<uint32_type>		u32_string;





/**
   Unicode を軸にしたコード変換と変換用リソースの管理。
 **/
class CBaseUSConv {

public:
	/**
	   iconv ディスクリプタリソース管理用ベースクラス。
	**/
	class IConvT {
		iconv_t		m_cd;
		std::string	m_tocode;
		std::string m_fromcode;
	public:
		IConvT() : m_cd((iconv_t)(-1)) {}
		~IConvT() { close(); }

		bool open(const char *tocode, const char *fromcode) {
			if( is_open() ) {
				if( m_tocode == tocode && m_fromcode == fromcode )
					return true;
				close();
			}
			m_cd = iconv_open(tocode,fromcode);
			if (m_cd != (iconv_t)(-1)) {
				m_fromcode = fromcode;
				m_tocode = tocode;
				return true;
			}
			return false;
		}

		bool is_open() { return m_cd != (iconv_t)(-1); }

		void close() {
			if( is_open() )
				iconv_close(m_cd);
			m_cd = (iconv_t)(-1);
			m_tocode.clear();
			m_fromcode.clear();
		}

		operator iconv_t() { return m_cd; }
	};

public:
	/**
	   MBCS型の文字列からWideChar型の文字列への変換。
	   （iconv_t インターフェース）
	**/
	template<typename WideChar>
	bool mbcstowcstring(
		iconv_t cd,
		const std::string&				mbcstr,
		std::basic_string<WideChar>&	wstr)
	{
		const char *inbuf = &mbcstr[0];
		size_t inbytesleft = mbcstr.size();

		wstr.resize(inbytesleft);
		char *outbuf = reinterpret_cast<char *>(&wstr[0]);
		char *const outbuf_begin = outbuf;
		size_t outbytesleft = wstr.size() * sizeof(WideChar);

		bool error = false;
		iconv( cd, NULL, 0, NULL, 0 ); /* マルチバイトシーケンスをリセット */
		while( inbytesleft > 0 ) {
			if( iconv( cd, CONST_CAST(char**,&inbuf), &inbytesleft, &outbuf, &outbytesleft ) == (size_t)-1 ) {
				error = true;
				/*エラーのあったところを飛ばす。
				  （シーケンスが復帰するまで連続するかも知れない） */
				inbuf ++;
				inbytesleft --;
				/* マーカーを入れておく */
				WideChar	error_marker = '?';
				std::memcpy(outbuf,&error_marker,sizeof(error_marker));
				outbuf += sizeof(error_marker);
				outbytesleft -= sizeof(error_marker);
			}
		}

		size_t	wlen = (outbuf - outbuf_begin) / sizeof(WideChar);
		wstr.resize(wlen);
		return !error;
	}

	/**
	   MBCS型の文字列からWideChar型の文字列への変換。
	   （IConvT& インターフェース）
	**/
	template<typename WideChar>
	bool ConvToWCS(IConvT& ict,
				   const char *mbcsname,
				   const char *wcsname,
				   const std::string& mbcstr,
				   std::basic_string<WideChar>& wstr)
	{
		if( !ict.open(wcsname,mbcsname) )
			return false;
		return mbcstowcstring<WideChar>( ict, mbcstr, wstr );
	}

	/**
	   WideChar型の文字列からMBCS型の文字列への変換。
	   （iconv_t インターフェース）
	**/
	template<typename WideChar>
	bool wcstombcstring(
		iconv_t cd,
		const std::basic_string<WideChar>& wstr,
		std::string& mbcstr)
	{
		const char *inbuf = reinterpret_cast<const char *>(&wstr[0]);
		size_t inbytesleft = wstr.size() * sizeof(WideChar);

		mbcstr.resize(wstr.size()*6 /*UTF-8 worst case */);
		char *outbuf = &mbcstr[0];
		size_t outbytesleft = mbcstr.size();

		bool error = false;
		iconv( cd, NULL, 0, NULL, 0 ); /* マルチバイトシーケンスをリセット */
		while( inbytesleft > 0 ) {
			if( iconv( cd, CONST_CAST(char**,&inbuf), &inbytesleft, &outbuf, &outbytesleft ) == (size_t)-1 ) {
				error = true;
				/*エラーのあったところを飛ばす。
				  （シーケンスが復帰するまで連続するかも知れない） */
				inbuf ++;
				inbytesleft --;
				/* マーカーを入れておく */
				*outbuf = '?';
				outbuf ++;
				outbytesleft --;
			}
		}
		size_t mblen = outbuf - &mbcstr[0];
		mbcstr.resize(mblen);
		return !error;
	}


	/**
	   WideChar型の文字列からMBCS型の文字列への変換。
	   （IConvT& インターフェース）
	**/
	template<typename WideChar>
	bool ConvToMBCS(IConvT& ict,
					const char *wcsname,
					const char *mbcsname,
					const std::basic_string<WideChar>& wstr,
					std::string& mbcstr)
	{
		if( !ict.open(mbcsname,wcsname) )
			return false;
		return wcstombcstring<WideChar>( ict, wstr, mbcstr );
	}


	/**
	   WideChar型の文字列同士の変換。
	   （iconv_t インターフェース）
	**/
	template<typename WideChar1,typename WideChar2>
	bool wcstowcstring(
		iconv_t cd,
		const std::basic_string<WideChar1>&		wstr1,
		std::basic_string<WideChar2>&			wstr2)
	{
		const char *inbuf = reinterpret_cast<const char *>(&wstr1[0]);
		size_t inbytesleft = wstr1.size() * sizeof(WideChar1);

		wstr2.resize(wstr1.size()*2); /* utf-32→utf-16で全部サロゲートペアになる
										 場合がワーストケース */
		char *outbuf = reinterpret_cast<char *>(&wstr2[0]);
		char *const outbuf_begin = outbuf;
		size_t outbytesleft = wstr2.size() * sizeof(WideChar2);

		bool error = false;
		iconv( cd, NULL, 0, NULL, 0 ); /* マルチバイトシーケンスをリセット */
		while( inbytesleft > 0 ) {
			if( iconv( cd, CONST_CAST(char **,&inbuf), &inbytesleft, &outbuf, &outbytesleft ) == (size_t)-1 ) {
				error = true;
				/*エラーのあったところを飛ばす。
				  （シーケンスが復帰するまで連続するかも知れない） */
				inbuf ++;
				inbytesleft --;
				/* マーカーを入れておく */
				WideChar2	error_marker = '?';
				std::memcpy(outbuf,&error_marker,sizeof(error_marker));
				outbuf += sizeof(error_marker);
				outbytesleft -= sizeof(error_marker);
			}
		}

		size_t	wlen = (outbuf - outbuf_begin) / sizeof(WideChar2);
		wstr2.resize(wlen);
		return !error;
	}

	/**
	   WideChar型の文字列同士の変換。
	   （IConvT& インターフェース）
	**/
	template<typename WideChar1,typename WideChar2>
	bool ConvWCS(IConvT& ict,
				 const char *name1,
				 const char *name2,
				 const std::basic_string<WideChar1>& wstr1,
				 std::basic_string<WideChar2>& 		 wstr2)
	{
		if( !ict.open(name2,name1) )
			return false;
		return wcstowcstring<WideChar1,WideChar2>( ict, wstr1, wstr2 );
	}

	/**
	   MBCS文字列同士の変換。
	   （iconv_t インターフェース）
	**/
	bool mbcstombcstring(
		iconv_t cd,
		const std::string&		str1,
		std::string&			str2)
	{
		const char *inbuf = &str1[0];
		size_t inbytesleft = str1.size();

		str2.resize(str1.size()*3); /* 2byte漢字→utf-8で6byteになる場合がワーストケース */
		char *outbuf = &str2[0];
		char *const outbuf_begin = outbuf;
		size_t outbytesleft = str2.size();

		bool error = false;
		iconv( cd, NULL, 0, NULL, 0 ); /* マルチバイトシーケンスをリセット */
		while( inbytesleft > 0 ) {
			if( iconv( cd, CONST_CAST(char**,&inbuf), &inbytesleft, &outbuf, &outbytesleft ) == (size_t)-1 ) {
				if( *inbuf == '\\' ) {
					/** UTF-8 の Ascii領域の backslash はSHIFT_JIS（CP932も）の
						半角 yen 記号には変換出来ないらしい。不思議な仕様だが、ス
						ルーしておけばよいことなので、そのようにする。
						ちなみに、SHIFT_JIS の yen 記号は UTF-8 では全角のYEN記号
						に変換される。これもよく分からない仕様だが、こちらは逆変
						換すれば元に戻るのでたぶん実用上問題にはならない。
					**/
					*outbuf = *inbuf;
				}
				else {
					/* 上記以外の変換出来なかった文字の位置にはマーカーを入れておく */
					*outbuf = '?';
					error = true;
				}
				outbuf ++;
				outbytesleft --;

					/*エラーのあったところを飛ばす。
					  （シーケンスが復帰するまで連続するかも知れない） */
				inbuf ++;
				inbytesleft --;
			}
		}

		size_t	len = outbuf - outbuf_begin;
		str2.resize(len);
		return !error;
	}

	/**
	   MBCS文字列同士の変換。
	   （IConvT& インターフェース）
	**/
	bool ConvMBCS(IConvT& ict,
				  const char *name1,
				  const char *name2,
				  const std::string& str1,
				  std::string& 		str2)
	{
		if( !ict.open(name2,name1) )
			return false;
		return mbcstombcstring( ict, str1, str2 );
	}

};

} /* namespace al */

#endif /*ALBASEUSCONV_ICONV_HPP_*/
