/**@file
   文字コード変換のためのユーティリティクラス（CSysCharSet）のメンバ関数の実装。
   全体から参照して使えるデフォルトインスタンスと、取得のための関数も保持する。

**/   

#include "ALSysCharSet.hpp"

namespace al {

namespace {
#if defined(_WINDOWS)
CSysCharSet		sys_char_set(CSysCharSet::ST_Windows);
#elif defined(_LINUX)
CSysCharSet		sys_char_set(CSysCharSet::ST_Linux_UTF8);
#elif defined(_MAC)
CSysCharSet		sys_char_set(CSysCharSet::ST_Mac);
#else
CSysCharSet		sys_char_set;
#endif



}

CSysCharSet& CSysCharSet::GetDefSysCharSet() { return sys_char_set; }

/*----------*/

bool CSysCharSet::ConvertToSysMBCS(
	const std::wstring& inner_str, 
	std::string&		sys_str   
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvToMBCS( cvt_16_to_sjis, UTF16_Encoding(), SJIS_Encoding(),
						  inner_str, sys_str );
	case ST_Linux_EUCJP:
		return ConvToMBCS( cvt_32_to_eucjp, UTF32_Encoding(), EUCJP_Encoding(),
						  inner_str, sys_str );
	case ST_Linux_UTF8:
	case ST_Mac: 
		return ConvToMBCS( cvt_32_to_utf8, UTF32_Encoding(), UTF8_Encoding(),
						  inner_str, sys_str );
	default:
		break;
	}
	return false;
}

bool CSysCharSet::ConvertToSysMBCS(
	const std::string&	inner_str, 
	std::string&		sys_str   
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvMBCS( cvt_utf8_to_sjis, UTF8_Encoding(), SJIS_Encoding(),
						inner_str, sys_str );
	case ST_Linux_EUCJP:
		return ConvMBCS( cvt_utf8_to_eucjp, UTF8_Encoding(), EUCJP_Encoding(),
						  inner_str, sys_str );
	case ST_Linux_UTF8:
	case ST_Mac: 
		sys_str = inner_str;
		return true;
	default:
		break;
	}
	return false;
}

/*----------*/

bool CSysCharSet::ConvertFromSysMBCS(
	const std::string&		sys_str,		
	std::wstring&			inner_str		
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvToWCS( cvt_16_from_sjis, SJIS_Encoding(), UTF16_Encoding(),
						  sys_str, inner_str );
	case ST_Linux_EUCJP:
		return ConvToWCS( cvt_32_from_eucjp, EUCJP_Encoding(), UTF32_Encoding(),
						  sys_str, inner_str );
	case ST_Linux_UTF8:
	case ST_Mac: 
		return ConvToWCS( cvt_32_from_utf8, UTF8_Encoding(), UTF32_Encoding(),
						  sys_str, inner_str );
	default:
		break;
	}
	return false;
}

bool CSysCharSet::ConvertFromSysMBCS(
	const std::string&		sys_str,		
	std::string&			inner_str		
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvMBCS( cvt_utf8_from_sjis, SJIS_Encoding(), UTF8_Encoding(),
						  sys_str, inner_str );
	case ST_Linux_EUCJP:
		return ConvMBCS( cvt_utf8_from_eucjp, EUCJP_Encoding(), UTF8_Encoding(),
						  sys_str, inner_str );
	case ST_Linux_UTF8:
	case ST_Mac: 
		inner_str = sys_str;
		return true;
	default:
		break;
	}
	return false;
}

/*----------*/

bool CSysCharSet::ConvertToSysWCS(
	const std::string&	inner_str, /**< UTF-8文字列 */
	std::wstring&		sys_str   /**< ワイドシステム文字列 */
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvToWCS( cvt_16_from_utf8, UTF8_Encoding(), UTF16_Encoding(),
						  inner_str, sys_str );
	case ST_Linux_EUCJP:
	case ST_Linux_UTF8:
	case ST_Mac: 
		return ConvToWCS( cvt_32_from_utf8, UTF8_Encoding(), UTF32_Encoding(),
						  inner_str, sys_str );
	default:
		break;
	}
	return false;
}

bool CSysCharSet::ConvertFromSysWCS(
	const std::wstring&		sys_str,		/**< ワイドシステム文字列 */
	std::string&			inner_str		/**< UTF-8文字列 */
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvToMBCS( cvt_16_to_utf8, UTF16_Encoding(), UTF8_Encoding(),
						   sys_str, inner_str );
	case ST_Linux_EUCJP:
	case ST_Linux_UTF8:
	case ST_Mac: 
		return ConvToMBCS( cvt_32_to_utf8, UTF32_Encoding(), UTF8_Encoding(),
						   sys_str, inner_str );
	default:
		break;
	}
	return false;
}

/*----------*/
		
bool CSysCharSet::ConvertToU32WCS(
	const std::wstring& inner_str, 
	u32_string&		u32_str	   
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvWCS( cvt_16_to_32, UTF16_Encoding(), UTF32_Encoding(),
						inner_str, u32_str );
	case ST_Linux_EUCJP:
	case ST_Linux_UTF8:
	case ST_Mac: 
		u32_str.resize(inner_str.size());
		copy(inner_str.begin(),inner_str.end(),u32_str.begin());
		return true;
	default:
		break;
	}
	return false;
}

bool CSysCharSet::ConvertToU32WCS(
	const std::string&	inner_str, 
	u32_string&		u32_str	   
	)
{
	return ConvToWCS( cvt_32_from_utf8, UTF8_Encoding(), UTF32_Encoding(),
					  inner_str, u32_str );
}

/*----------*/

bool CSysCharSet::ConvertFromU32WCS(
	const u32_string&		u32_str,
	std::wstring& 			inner_str
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		return ConvWCS( cvt_32_to_16, UTF32_Encoding(), UTF16_Encoding(),
						u32_str, inner_str );
	case ST_Linux_EUCJP:
	case ST_Linux_UTF8:
	case ST_Mac: 
		inner_str.resize(inner_str.size());
		copy(u32_str.begin(),u32_str.end(),inner_str.begin());
		return true;
	default:
		break;
	}
	return false;
}

bool CSysCharSet::ConvertFromU32WCS(
	const u32_string&		u32_str,		
	std::string&			inner_str		
	)
{
	return ConvToMBCS( cvt_32_to_utf8, UTF32_Encoding(), UTF8_Encoding(),
					  u32_str, inner_str );
}

/*----------*/

bool CSysCharSet::ConvertToU16WCS(
	const std::wstring& inner_str, 
	u16_string&		u16_str	   
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		u16_str.resize(inner_str.size());
		copy(inner_str.begin(),inner_str.end(),u16_str.begin());
		return true;
	case ST_Linux_EUCJP:
	case ST_Linux_UTF8:
	case ST_Mac: 
		return ConvWCS( cvt_32_to_16, UTF32_Encoding(), UTF16_Encoding(),
						inner_str, u16_str );
	default:
		break;
	}
	return false;
}

bool CSysCharSet::ConvertToU16WCS(
	const std::string& inner_str, 
	u16_string&		u16_str	   
	)
{
	return ConvToWCS( cvt_16_from_utf8, UTF8_Encoding(), UTF16_Encoding(),
					  inner_str, u16_str );
}

/*----------*/

bool CSysCharSet::ConvertFromU16WCS(
	const u16_string&		u16_str,		
	std::wstring&			inner_str		
	)
{
	switch( m_system_type ) {
	case ST_Windows:
		inner_str.resize(u16_str.size());
		copy(u16_str.begin(),u16_str.end(),inner_str.begin());
		return false;
	case ST_Linux_EUCJP:
	case ST_Linux_UTF8:
	case ST_Mac: 
		return ConvWCS( cvt_16_to_32, UTF16_Encoding(), UTF32_Encoding(),
						u16_str, inner_str );
	default:
		break;
	}
	return false;
}

bool CSysCharSet::ConvertFromU16WCS(
	const u16_string&		u16_str,		
	std::string&			inner_str		
	)
{
	return ConvToMBCS( cvt_16_to_utf8, UTF16_Encoding(), UTF8_Encoding(),
					  u16_str, inner_str );
}

} /* namespace al */
