/* -*- tab-width:4 -*- */
#pragma once
#if !defined(ALSYSCHARSET_HPP_)
#define ALSYSCHARSET_HPP_

/**@file
   文字コード変換のためのユーティリティクラス（CSysCharSet）の宣言。
   下記のシステム毎の差を吸収するためのもの。

   - wstring(wchar_t) の型
     -- Windows= UTF-16le、Linux=UTF-32le
   - ファイルシステム向け文字コード(MBCS)
     -- Windows=Shift-JIS、Linux=UTF-8 or EUC-JP
   - ファイルシステム向け文字コード(wchar_t)
     -- Windows=UTF-16le、Linux=なし

   ８bit内部文字コードは UTF-8 MBCSで決め打ちとし、必要な変換関数を規定する。

   @note UTF-16/UTF-32 と std::string との変換はシステムの文字コードにはよらず実
   行できるので、常にデフォルト文字セットで利用して構わない。

**/

#include <string>
#include <algorithm>
#include "AlUSConvJP.hpp"

namespace al {

/**
   稼働中のシステムで用いられる文字コードを認識し、
   必要な変換インターフェースを選択するためのクラス。
 **/
class CSysCharSet : public CUSConvJP {
public:
								/** システムタイプを示す定数 */
	enum SystemType {
		ST_Unknown,				/**< まだ不明 */
		ST_Windows,				/**< Windows */
		ST_Linux_EUCJP,			/**< EUC-JP ベースのLinux */
		ST_Linux_UTF8,			/**< UTF-8 ベースのLinux */
		ST_Mac,					/**< Macintosh (Intel 以降のみ) */
		MAX_SystemType
	};

public:
	/** デフォルトコンストラクタではシステムは指定しない */
	CSysCharSet()
		: m_system_type(ST_Unknown)
	{
	}

	/** 最初からシステムを明示して生成するインターフェース */
	explicit CSysCharSet(SystemType system_type)
		: m_system_type(system_type)
	{
	}

	/** デフォルトで使えるインスタンスの取得 */
	static CSysCharSet& GetDefSysCharSet();

	/** デフォルトで使えるインスタンスにSystemTypeを設定。
		@attention デフォルトでは SystemType は ST_Unknown */
	static void SetDefSystemType(SystemType system_type) {
		GetDefSysCharSet().SetSystemType(system_type);
	}

	/** クラスのインスタンスに SystemType を設定 */
	void SetSystemType( SystemType system_type )
	{
		m_system_type = system_type;
	}

	/** MBCSシステム文字列への（wstring → string）変換 */
	bool ConvertToSysMBCS(
		const std::wstring& inner_str, /**< 内部形式ワイド文字列 */
		std::string&		sys_str	   /**< システム形式MBCS文字列 */
		);

	/** MBCSシステム文字列への（string → string）変換 */
	bool ConvertToSysMBCS(
		const std::string&	inner_str, /**< UTF-8文字列 */
		std::string&		sys_str   /**< システム形式MBCS文字列 */
		);

	/** MBCSシステム文字列からの（string → wstring）変換 */
	bool ConvertFromSysMBCS(
		const std::string&		sys_str,		/**< システム形式MBCS文字列 */
		std::wstring&			inner_str		/**< 内部形式ワイド文字列 */
		);

	/** MBCSシステム文字列からの（string → string）変換 */
	bool ConvertFromSysMBCS(
		const std::string&		sys_str,		/**< システム形式MBCS文字列 */
		std::string&			inner_str		/**< UTF-8文字列 */
		);

	/** ワイドシステム文字列への（string → wstring）変換 */
	bool ConvertToSysWCS(
		const std::string&	inner_str, /**< UTF-8文字列 */
		std::wstring&		sys_str	   /**< ワイドシステム文字列 */
		);

	/** ワイドシステム文字列からの（wstring → string）変換 */
	bool ConvertFromSysWCS(
		const std::wstring&		sys_str,		/**< ワイドシステム文字列 */
		std::string&			inner_str		/**< UTF-8文字列 */
		);
		
	/** UTF-32文字列への（wstring → u32_string）変換 */
	bool ConvertToU32WCS(
		const std::wstring& inner_str, /**< 内部形式ワイド文字列 */
		u32_string&		u32_str	   /**< UTF-32文字列 */
		);

	/** UTF-32文字列への（string → u32_string）変換 */
	bool ConvertToU32WCS(
		const std::string&  inner_str, /**< UTF-8文字列 */
		u32_string&		u32_str	   /**< UTF-32文字列 */
		);

	/** UTF-32文字列からの（u32_string → wstring）変換 */
	bool ConvertFromU32WCS(
		const u32_string&		u32_str,		/**< UTF-32文字列 */
		std::wstring&			inner_str		/**< 内部形式ワイド文字列 */
		);

	/** UTF-32文字列からの（u32_string → string）変換 */
	bool ConvertFromU32WCS(
		const u32_string&		u32_str,		/**< UTF-32文字列 */
		std::string&			inner_str		/**< UTF-8文字列 */
		);
		
	/** UTF-16文字列への（wstring → u16_string）変換 */
	bool ConvertToU16WCS(
		const std::wstring& inner_str, /**< 内部形式ワイド文字列 */
		u16_string&		u16_str	   /**< UTF-16文字列 */
		);

	/** UTF-16文字列への（string → u16_string）変換 */
	bool ConvertToU16WCS(
		const std::string&  inner_str, /**< UTF-8文字列 */
		u16_string&		u16_str	   /**< UTF-16文字列 */
		);

	/** UTF-16文字列からの（u16_string → wstring）変換 */
	bool ConvertFromU16WCS(
		const u16_string&		u16_str,		/**< UTF-16文字列 */
		std::wstring&			inner_str		/**< 内部形式ワイド文字列 */
		);

	/** UTF-16文字列からの（u16_string → string）変換 */
	bool ConvertFromU16WCS(
		const u16_string&		u16_str,		/**< UTF-16文字列 */
		std::string&			inner_str		/**< UTF-8文字列 */
		);

    /** std::string からの、std::stringへの変換（窓口）
        @attention このインターフェースは templateで使うことを想定している。 */
    static const std::string ConvertToStdString( const std::string& std_str ) {
        return std_str;
    }

    /** UTF-16 からの、std::stringへの変換（窓口）
        @attention このインターフェースは templateで使うことを想定している。 */
    static const std::string ConvertToStdString( const u16_string& u16_str ) {
        std::string	std_string;
        GetDefSysCharSet().ConvertFromU16WCS(u16_str,std_string);
        return std_string;
    }

    /** UTF-32 からの、std::stringへの変換（窓口）
        @attention このインターフェースは templateで使うことを想定している。 */
    static const std::string ConvertToStdString( const u32_string& u32_str ) {
        std::string	std_string;
        GetDefSysCharSet().ConvertFromU32WCS(u32_str,std_string);
        return std_string;
    }

    /** std::string から std::stringへの変換（窓口）
        @attention このインターフェースは templateで使うことを想定している。 */
    static const std::string ConvertFromStdString( const std::string& std_str ) {
        return std_str;
    }

    /** std::string から UTF-16 への変換（窓口）
        @attention このインターフェースは templateで使うことを想定している。 */
    static const u16_string ConvertFromStdString(
        const std::string& std_string,
        const u16_string&
        )
    {
        u16_string	u16_string;
        GetDefSysCharSet().ConvertToU16WCS(std_string,u16_string);
        return u16_string;
    }

    /** std::string から UTF-32 への変換（窓口）
        @attention このインターフェースは templateで使うことを想定している。 */
    static const u32_string ConvertFromStdString(
        const std::string& std_string,
        const u32_string&
        )
    {
        u32_string	u32_string;
        GetDefSysCharSet().ConvertToU32WCS(std_string,u32_string);
        return u32_string;
    }


private:
								/** システムタイプ */
	SystemType	m_system_type;

};

} /* namespace al */

#endif /*ALSYSCHARSET_HPP_*/
