#pragma once
#if !defined(ALUSCONVJP_HPP_)
#define ALUSCONVJP_HPP_

/**@file
   代表的な日本語コードとの変換処理クラス（CUSConvJP）の宣言。
 **/

#if defined(USE_ICONV)
#include "ALBaseUSConv_iconv.hpp"
#else
#include "ALBaseUSConv_icu.hpp"
#endif


namespace al {

class CUSConvJP : public CBaseUSConv {

public:
#if defined(USE_ICONV)
typedef IConvT ConvT;
#else
typedef UConvT ConvT;
#endif

private:
	const char*			m_sjis_encoding;
	const char*			m_eucjp_encoding;
	const char*			m_utf8_encoding;
	const char*	 		m_utf16_encoding;
	const char*			m_utf32_encoding;

	bool IsBigEndian() {
		const wchar_t	a = L'a';
		return (*reinterpret_cast<const char *>(&a) == '\0');
	}

protected:
	const char* SJIS_Encoding() { return m_sjis_encoding; }
	const char* EUCJP_Encoding() { return m_eucjp_encoding; }
	const char* UTF8_Encoding() { return m_utf8_encoding; }

	const char * UTF16_Encoding() {
		if( m_utf16_encoding == NULL )
			m_utf16_encoding = (IsBigEndian() ? "UTF-16BE" : "UTF-16LE");
		return m_utf16_encoding;
	}

	const char * UTF32_Encoding() {
		if( m_utf32_encoding == NULL )
			m_utf32_encoding = (IsBigEndian() ? "UTF-32BE" : "UTF-32LE");
		return m_utf32_encoding;
	}

	ConvT	cvt_16_to_utf8;
	ConvT	cvt_16_to_sjis;
	ConvT	cvt_16_to_eucjp;
	ConvT	cvt_16_from_utf8;
	ConvT	cvt_16_from_sjis;
	ConvT	cvt_16_from_eucjp;
	ConvT	cvt_32_to_utf8;
	ConvT	cvt_32_to_sjis;
	ConvT	cvt_32_to_eucjp;
	ConvT	cvt_32_from_utf8;
	ConvT	cvt_32_from_sjis;
	ConvT	cvt_32_from_eucjp;

	ConvT	cvt_16_to_32;
	ConvT	cvt_32_to_16;

	ConvT	cvt_utf8_to_sjis;
	ConvT	cvt_utf8_to_eucjp;
	ConvT	cvt_utf8_from_sjis;
	ConvT	cvt_utf8_from_eucjp;

public:

	/** CP932 と SHIFT_JIS は一長一短だが、とりあえず CP932 の方が害が少ない。
		UTF-16/UTF-32 のBE/LEは実際の文字を見て判定する。 */
	CUSConvJP() :
		m_sjis_encoding("CP932"),
		m_eucjp_encoding("EUC-JP"),
		m_utf8_encoding("UTF-8"),
		m_utf16_encoding(NULL),
		m_utf32_encoding(NULL) {}

	/** UTF-16 -> utf-8 への変換。 **/
	bool CodeConvUtf16ToUtf8(
		const u16_string& 	str_utf16,
		std::string&			str_utf8
		)
	{
		return ConvToMBCS( cvt_16_to_utf8, UTF16_Encoding(), UTF8_Encoding(), str_utf16, str_utf8 );
	}

	/** UTF-16 -> sjis への変換。 **/
	bool CodeConvUtf16ToSJIS(
		const u16_string& 	str_utf16,
		std::string&			str_sjis
		)
	{
		return ConvToMBCS( cvt_16_to_sjis, UTF16_Encoding(), SJIS_Encoding(), str_utf16, str_sjis );
	}

	/** UTF-16 -> euc-jp への変換。 **/
	bool CodeConvUtf16ToEucJp(
		const u16_string& 	str_utf16,
		std::string&			str_eucjp
		)
	{
		return ConvToMBCS( cvt_16_to_eucjp, UTF16_Encoding(), EUCJP_Encoding(), str_utf16, str_eucjp );
	}

	/** UTF-32 -> utf-8 への変換。 **/
	bool CodeConvUtf32ToUtf8(
		const u32_string& 	str_utf32,
		std::string&			str_utf8
		)
	{
		return ConvToMBCS( cvt_32_to_utf8, UTF32_Encoding(), UTF8_Encoding(), str_utf32, str_utf8 );
	}

	/** UTF-32 -> sjis への変換。 **/
	bool CodeConvUtf32ToSJIS(
		const u32_string& 	str_utf32,
		std::string&			str_sjis
		)
	{
		return ConvToMBCS( cvt_32_to_sjis, UTF32_Encoding(), SJIS_Encoding(), str_utf32, str_sjis );
	}


	/** UTF-32 -> euc-jp への変換。 **/
	bool CodeConvUtf32ToEucJp(
		const u32_string& 	str_utf32,
		std::string&			str_eucjp
		)
	{
		return ConvToMBCS( cvt_32_to_eucjp, UTF32_Encoding(), EUCJP_Encoding(), str_utf32, str_eucjp );
	}

	/** utf-8 -> UTF-16 への変換。 **/
	bool CodeConvUtf8ToUtf16(
		const std::string&		str_utf8,
		u16_string& 			str_utf16
		)
	{
		return ConvToWCS( cvt_16_from_utf8, UTF8_Encoding(), UTF16_Encoding(), str_utf8, str_utf16 );
	}

	/** sjis -> UTF-16 への変換。 **/
	bool CodeConvSJISToUtf16(
		const std::string&		str_sjis,
		u16_string& 			str_utf16
		)
	{
		return ConvToWCS( cvt_16_from_sjis, SJIS_Encoding(), UTF16_Encoding(), str_sjis, str_utf16 );
	}

	/** euc-jp -> UTF-16 への変換。 **/
	bool CodeConvEucJpToUtf16(
		const std::string&		str_eucjp,
		u16_string& 			str_utf16
		)
	{
		return ConvToWCS( cvt_16_from_eucjp, EUCJP_Encoding(), UTF16_Encoding(), str_eucjp, str_utf16 );
	}

	/** utf-8 -> UTF-32 への変換。 **/
	bool CodeConvUtf8ToUtf32(
		const std::string&		str_utf8,
		u32_string& 			str_utf32
		)
	{
		return ConvToWCS( cvt_32_from_utf8, UTF8_Encoding(), UTF32_Encoding(), str_utf8, str_utf32 );
	}

	/** sjis -> UTF-32 への変換。 **/
	bool CodeConvSJISToUtf32(
		const std::string&		str_sjis,
		u32_string& 			str_utf32
		)
	{
		return ConvToWCS( cvt_32_from_sjis, SJIS_Encoding(), UTF32_Encoding(), str_sjis, str_utf32 );
	}

	/** euc-jp -> UTF-32 への変換。 **/
	bool CodeConvEucJpToUtf32(
		const std::string&		str_eucjp,
		u32_string& 			str_utf32
		)
	{
		return ConvToWCS( cvt_32_from_eucjp, EUCJP_Encoding(), UTF32_Encoding(), str_eucjp, str_utf32 );
	}

	/** UTF-16 -> UTF-32 への変換。 **/
	bool CodeConvUtf16ToUtf32(
		const u16_string&		str_utf16,
		u32_string& 			str_utf32
		)
	{
		return ConvWCS( cvt_16_to_32, UTF16_Encoding(), UTF32_Encoding(), str_utf16, str_utf32 );
	}

	/** UTF-32 -> UTF-16 への変換。 **/
	bool CodeConvUtf32ToUtf16(
		const u32_string& 	str_utf32,
		u16_string&			str_utf16
		)
	{
		return ConvWCS( cvt_32_to_16, UTF32_Encoding(), UTF16_Encoding(), str_utf32, str_utf16 );
	}

	/** UTF-8 -> sjis への変換。 **/
	bool CodeConvUtf8ToSJIS(
		const std::string& 		str_utf8,
		std::string&			str_sjis
		)
	{
		return ConvMBCS( cvt_utf8_to_sjis, UTF8_Encoding(), SJIS_Encoding(), str_utf8, str_sjis );
	}

	/** UTF-8 -> euc-jp への変換。 **/
	bool CodeConvUtf8ToEucJp(
		const std::string& 		str_utf8,
		std::string&			str_eucjp
		)
	{
		return ConvMBCS( cvt_utf8_to_eucjp, UTF8_Encoding(), EUCJP_Encoding(), str_utf8, str_eucjp );
	}

	/** sjis -> UTF-8 への変換。 **/
	bool CodeConvSJISToUtf8(
		const std::string&		str_sjis,
		std::string& 			str_utf8
		)
	{
		return ConvMBCS( cvt_utf8_from_sjis, SJIS_Encoding(), UTF8_Encoding(), str_sjis, str_utf8 );
	}

	/** euc-jp -> UTF-8 への変換。 **/
	bool CodeConvEucJpToUtf8(
		const std::string&		str_eucjp,
		std::string& 			str_utf8
		)
	{
		return ConvMBCS( cvt_utf8_from_eucjp, EUCJP_Encoding(), UTF8_Encoding(), str_eucjp, str_utf8 );
	}


    /** 変換ロジックのテストルーチン。順番に変換していって、最後に元に戻ることを確認する。
        正常終了時は 0を返す。 異常終了時は0以外の正の値（エラーになった場所を示す）。 */
    int CodeConvTest();

};

} /* namespace al */


#endif /*ALUSCONVJP_HPP_*/
